==> sposh-core-3.3.0-20120310.124013-15-sources.jar: cz/cuni/amis/pogamut/sposh/engine/DCExecutor.java <==
package cz.cuni.amis.pogamut.sposh.engine;

import cz.cuni.amis.pogamut.sposh.engine.timer.SystemClockTimer;
import cz.cuni.amis.pogamut.sposh.engine.timer.ITimer;
import cz.cuni.amis.pogamut.sposh.elements.DriveCollection;
import cz.cuni.amis.pogamut.sposh.elements.DriveElement;
import cz.cuni.amis.pogamut.sposh.elements.PoshPlan;
import cz.cuni.amis.pogamut.sposh.engine.PoshEngine.EvaluationResult;
import cz.cuni.amis.pogamut.sposh.executor.IWorkExecutor;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Logger;

/**
 * This class is responsible for valuating the DriveCollection element.
 *
 * @author HonzaH
 */
final class DCExecutor extends AbstractExecutor {

    private SenseListExecutor goalExecutor;
    List<DEExecutor> deExecutors = new ArrayList<DEExecutor>();
    private ITimer timer;
    private TriggerResult triggerResult;

    /**
     * Create DCExecutor with SystemClockTimer.
     *
     * @param dc
     * @param ctx
     */
    protected DCExecutor(PoshPlan plan, VariableContext ctx) {
        this(plan, ctx, new SystemClockTimer(), null);
    }

    protected DCExecutor(PoshPlan plan, VariableContext ctx, ITimer timer, Logger log) {
        super(ctx, log);

        DriveCollection dc = plan.getDriveCollection();
        this.timer = timer;
        this.goalExecutor = new SenseListExecutor(dc.getGoal(), ctx, log);

        for (DriveElement de : dc.getDrives()) {
            deExecutors.add(new DEExecutor(plan, de, ctx, log));
        }
    }

    public synchronized PoshEngine.EvaluationResultInfo fire(IWorkExecutor workExecuter) {
        FireResult.Type resultType = null;
        triggerResult = goalExecutor.fire(workExecuter, false);
        if (triggerResult.wasSuccess()) {
            return new PoshEngine.EvaluationResultInfo(EvaluationResult.GOAL_SATISFIED, FireResult.Type.FULFILLED);
        }
        // TODO: Rework whole negine, currently I need to clear trigger re4sults of DEExecutor every loop
        // because they would linger, although never fired in the loop
        for (DEExecutor deExecutor : deExecutors) {
            deExecutor.clearTriggerResult();
        }

        for (DEExecutor deExecutor : deExecutors) {
            if (deExecutor.isReady(timer.getTime(), workExecuter)) {
                resultType = deExecutor.fire(workExecuter, timer);
                return new PoshEngine.EvaluationResultInfo(EvaluationResult.ELEMENT_FIRED, resultType);
            }
        }
        return new PoshEngine.EvaluationResultInfo(EvaluationResult.NO_ELEMENT_FIRED, FireResult.Type.FAILED);
    }

    ElementStackTrace getStackForDE(int index) {
        return deExecutors.get(index).getStackTrace();
    }

    /**
     * Get ElementStackTrace for the drive element with name "name." If there
     * are two drive elements with same name, throw IllegalStateException.
     *
     * @param name name of drive element we are looking for
     * @return stacktrace for element if element with such name exists, else
     * null.
     */
    ElementStackTrace getStackForDE(String name) {
        DEExecutor result = null;
        for (DEExecutor de : deExecutors) {
            boolean equal = de.getName() == null ? name == null : de.getName().equals(name);
            if (equal) {
                if (result == null) {
                    result = de;
                } else {
                    throw new IllegalStateException("Two drive elements with name \"" + name + "\".");
                }
            }
        }
        if (result != null) {
            return result.getStackTrace();
        }

        return null;
    }

    // TODO: Change DC executor into an ElementExecutor
    public TriggerResult getTriggerResult() {
        return triggerResult;
    }
}

==> sposh-core-3.3.2-20121222.021614-98-sources.jar: cz/cuni/amis/pogamut/sposh/engine/DCExecutor.java <==
package cz.cuni.amis.pogamut.sposh.engine;

import cz.cuni.amis.pogamut.sposh.engine.timer.SystemClockTimer;
import cz.cuni.amis.pogamut.sposh.engine.timer.ITimer;
import cz.cuni.amis.pogamut.sposh.elements.DriveCollection;
import cz.cuni.amis.pogamut.sposh.elements.DriveElement;
import cz.cuni.amis.pogamut.sposh.elements.PoshPlan;
import cz.cuni.amis.pogamut.sposh.engine.PoshEngine.EvaluationResult;
import cz.cuni.amis.pogamut.sposh.executor.IWorkExecutor;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Logger;

/**
 * This class is responsible for valuating the DriveCollection element.
 *
 * @author HonzaH
 */
final class DCExecutor extends AbstractExecutor {

    private SenseListExecutor<DriveCollection> goalExecutor;
    List<DEExecutor> deExecutors = new ArrayList<DEExecutor>();
    private DEExecutor lastTriggeredDrive = null;
    private ITimer timer;
    private TriggerResult triggerResult;
    

    /**
     * Create DCExecutor with SystemClockTimer.
     *
     * @param dc
     * @param ctx
     */
    protected DCExecutor(PoshPlan plan, VariableContext ctx) {
        this(plan, ctx, new SystemClockTimer(), null);
    }

    protected DCExecutor(PoshPlan plan, VariableContext ctx, ITimer timer, Logger log) {
        super(ctx, log);

        DriveCollection dc = plan.getDriveCollection();
        this.timer = timer;
        this.goalExecutor = new SenseListExecutor<DriveCollection>(dc.getGoal(), ctx, log);

        for (DriveElement de : dc.getDrives()) {
            deExecutors.add(new DEExecutor(plan, de, ctx, log));
        }
    }

    public synchronized PoshEngine.EvaluationResultInfo fire(IWorkExecutor workExecuter) {
        FireResult.Type resultType = null;
        triggerResult = goalExecutor.fire(workExecuter, false);
        if (triggerResult.wasSuccess()) {
            return new PoshEngine.EvaluationResultInfo(EvaluationResult.GOAL_SATISFIED, FireResult.Type.FULFILLED);
        }
        // TODO: Rework whole engine, currently I need to clear trigger re4sults of DEExecutor every loop
        // because they would linger, although never fired in the loop
        for (DEExecutor deExecutor : deExecutors) {
            deExecutor.clearTriggerResult();
        }

        for (DEExecutor deExecutor : deExecutors) {
            if (deExecutor.isReady(timer.getTime(), workExecuter)) {
            	if (lastTriggeredDrive != null && lastTriggeredDrive != deExecutor) {
            		// DRIVE SWITCH
            		// Some drive that has more priority than "lastTriggeredDrive" has to fire
            		// => clean up the stack of deExecutor
            		lastTriggeredDrive.driveInterrupted();            		
            	}            	
            	lastTriggeredDrive = deExecutor;
                resultType = deExecutor.fire(workExecuter, timer);
                return new PoshEngine.EvaluationResultInfo(EvaluationResult.ELEMENT_FIRED, resultType);
            }
        }
        return new PoshEngine.EvaluationResultInfo(EvaluationResult.NO_ELEMENT_FIRED, FireResult.Type.FAILED);
    }
    
    int getDECount() {
    	return deExecutors.size();
    }
    
    String getDEName(int index) {
    	return deExecutors.get(index).getName();
    }

    ElementStackTrace getStackForDE(int index) {
        return deExecutors.get(index).getStackTrace();
    }

    /**
     * Get ElementStackTrace for the drive element with name "name." If there
     * are two drive elements with same name, throw IllegalStateException.
     *
     * @param name name of drive element we are looking for
     * @return stacktrace for element if element with such name exists, else
     * null.
     */
    ElementStackTrace getStackForDE(String name) {
        DEExecutor result = null;
        for (DEExecutor de : deExecutors) {
            boolean equal = de.getName() == null ? name == null : de.getName().equals(name);
            if (equal) {
                if (result == null) {
                    result = de;
                } else {
                    throw new IllegalStateException("Two drive elements with name \"" + name + "\".");
                }
            }
        }
        if (result != null) {
            return result.getStackTrace();
        }

        return null;
    }

    // TODO: Change DC executor into an ElementExecutor
    public TriggerResult getTriggerResult() {
        return triggerResult;
    }
}
